#!/usr/bin/perl


use Test::More;

my $class  = "Business::US::USPS::WebTools::TrackConfirm";
my $method = 'track';


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
use_ok( $class );


done_testing();
